@@ -1,12 +1,20 @@
 var tab1, tab2;
 
+function focus_in_navbar() {
+  var parent = document.activeElement.parentNode;
+  while (parent && parent.id != "nav-bar")
+    parent = parent.parentNode;
+
+  return (parent != null);
+}
+
 function test() {
   waitForExplicitFinish();
 
-  tab1 = gBrowser.addTab();
-  tab2 = gBrowser.addTab();
+  tab1 = gBrowser.addTab("about:blank", {skipAnimation: true});
+  tab2 = gBrowser.addTab("about:blank", {skipAnimation: true});
 
-  EventUtils.synthesizeMouse(tab1, 2, 2, {});
+  EventUtils.synthesizeMouseAtCenter(tab1, {});
   setTimeout(step2, 0);
 }
 
@@ -14,7 +22,7 @@ function step2()
 {
   isnot(document.activeElement, tab1, "mouse on tab not activeElement");
 
-  EventUtils.synthesizeMouse(tab1, 2, 2, {});
+  EventUtils.synthesizeMouseAtCenter(tab1, {});
   setTimeout(step3, 0);
 }
 
@@ -22,11 +30,18 @@ function step3()
 {
   isnot(document.activeElement, tab1, "mouse on tab again activeElement");
 
-  document.getElementById("searchbar").focus();
-  EventUtils.synthesizeKey("VK_TAB", { });
+  if (gNavToolbox.getAttribute("tabsontop") == "true") {
+    gURLBar.focus();
+    EventUtils.synthesizeKey("VK_TAB", {shiftKey: true});
+  } else {
+    document.getElementById("searchbar").focus();
+
+    while (focus_in_navbar())
+      EventUtils.synthesizeKey("VK_TAB", { });
+  }
   is(document.activeElement, tab1, "tab key to tab activeElement");
 
-  EventUtils.synthesizeMouse(tab1, 2, 2, {});
+  EventUtils.synthesizeMouseAtCenter(tab1, {});
   setTimeout(step4, 0);
 }
 
@@ -34,7 +49,7 @@ function step4()
 {
   is(document.activeElement, tab1, "mouse on tab while focused still activeElement");
 
-  EventUtils.synthesizeMouse(tab2, 2, 2, {});
+  EventUtils.synthesizeMouseAtCenter(tab2, {});
   setTimeout(step5, 0);
 }
 
@@ -45,7 +60,7 @@ function step5()
   is(document.activeElement, tab2, "mouse on another tab while focused still activeElement");
 
   content.focus();
-  EventUtils.synthesizeMouse(tab2, 2, 2, {button: 1, type: "mousedown"});
+  EventUtils.synthesizeMouseAtCenter(tab2, {button: 1, type: "mousedown"});
   setTimeout(step6, 0);
 }
 
