@@ -54,20 +54,21 @@ nsDOMMouseEvent::nsDOMMouseEvent(nsPresContext* aPresContext,
   // DOM event.
   
   if (aEvent) {
-    NS_ASSERTION(static_cast<nsMouseEvent*>(mEvent)->reason
-                 != nsMouseEvent::eSynthesized,
-                 "Don't dispatch DOM events from synthesized mouse events");
     mEventIsInternal = PR_FALSE;
   }
   else {
     mEventIsInternal = PR_TRUE;
     mEvent->time = PR_Now();
     mEvent->refPoint.x = mEvent->refPoint.y = 0;
+    static_cast<nsMouseEvent*>(mEvent)->inputSource = nsIDOMNSMouseEvent::MOZ_SOURCE_UNKNOWN;
   }
 
   switch (mEvent->eventStructType)
   {
     case NS_MOUSE_EVENT:
+      NS_ASSERTION(static_cast<nsMouseEvent*>(mEvent)->reason
+                   != nsMouseEvent::eSynthesized,
+                   "Don't dispatch DOM events from synthesized mouse events");
       mDetail = static_cast<nsMouseEvent*>(mEvent)->clickCount;
       break;
     default:
@@ -94,10 +95,12 @@ nsDOMMouseEvent::~nsDOMMouseEvent()
 NS_IMPL_ADDREF_INHERITED(nsDOMMouseEvent, nsDOMUIEvent)
 NS_IMPL_RELEASE_INHERITED(nsDOMMouseEvent, nsDOMUIEvent)
 
+DOMCI_DATA(MouseEvent, nsDOMMouseEvent)
+
 NS_INTERFACE_MAP_BEGIN(nsDOMMouseEvent)
   NS_INTERFACE_MAP_ENTRY(nsIDOMMouseEvent)
   NS_INTERFACE_MAP_ENTRY(nsIDOMNSMouseEvent)
-  NS_INTERFACE_MAP_ENTRY_CONTENT_CLASSINFO(MouseEvent)
+  NS_DOM_INTERFACE_MAP_ENTRY_CLASSINFO(MouseEvent)
 NS_INTERFACE_MAP_END_INHERITING(nsDOMUIEvent)
 
 NS_IMETHODIMP
@@ -116,6 +119,7 @@ nsDOMMouseEvent::InitMouseEvent(const nsAString & aType, PRBool aCanBubble, PRBo
     case NS_MOUSE_SCROLL_EVENT:
     case NS_DRAG_EVENT:
     case NS_SIMPLE_GESTURE_EVENT:
+    case NS_MOZTOUCH_EVENT:
     {
        static_cast<nsMouseEvent_base*>(mEvent)->relatedTarget = aRelatedTarget;
        static_cast<nsMouseEvent_base*>(mEvent)->button = aButton;
@@ -148,7 +152,7 @@ nsDOMMouseEvent::InitNSMouseEvent(const nsAString & aType, PRBool aCanBubble, PR
                                   PRInt32 aScreenY, PRInt32 aClientX, PRInt32 aClientY,
                                   PRBool aCtrlKey, PRBool aAltKey, PRBool aShiftKey,
                                   PRBool aMetaKey, PRUint16 aButton, nsIDOMEventTarget *aRelatedTarget,
-                                  float aPressure)
+                                  float aPressure, PRUint16 aInputSource)
 {
   nsresult rv = nsDOMMouseEvent::InitMouseEvent(aType, aCanBubble, aCancelable,
                                                 aView, aDetail, aScreenX, aScreenY,
@@ -157,6 +161,7 @@ nsDOMMouseEvent::InitNSMouseEvent(const nsAString & aType, PRBool aCanBubble, PR
   NS_ENSURE_SUCCESS(rv, rv);
 
   static_cast<nsMouseEvent_base*>(mEvent)->pressure = aPressure;
+  static_cast<nsMouseEvent_base*>(mEvent)->inputSource = aInputSource;
   return NS_OK;
 }
 
@@ -170,6 +175,7 @@ nsDOMMouseEvent::GetButton(PRUint16* aButton)
     case NS_MOUSE_SCROLL_EVENT:
     case NS_DRAG_EVENT:
     case NS_SIMPLE_GESTURE_EVENT:
+    case NS_MOZTOUCH_EVENT:
       *aButton = static_cast<nsMouseEvent_base*>(mEvent)->button;
       break;
     default:
@@ -192,6 +198,7 @@ nsDOMMouseEvent::GetRelatedTarget(nsIDOMEventTarget** aRelatedTarget)
     case NS_MOUSE_SCROLL_EVENT:
     case NS_DRAG_EVENT:
     case NS_SIMPLE_GESTURE_EVENT:
+    case NS_MOZTOUCH_EVENT:
       relatedTarget = static_cast<nsMouseEvent_base*>(mEvent)->relatedTarget;
       break;
     default:
@@ -294,6 +301,14 @@ nsDOMMouseEvent::GetMozPressure(float* aPressure)
   return NS_OK;
 }
 
+NS_IMETHODIMP
+nsDOMMouseEvent::GetMozInputSource(PRUint16* aInputSource)
+{
+  NS_ENSURE_ARG_POINTER(aInputSource);
+  *aInputSource = static_cast<nsMouseEvent_base*>(mEvent)->inputSource;
+  return NS_OK;
+}
+
 nsresult NS_NewDOMMouseEvent(nsIDOMEvent** aInstancePtrResult,
                              nsPresContext* aPresContext,
                              nsInputEvent *aEvent) 
