@@ -50,6 +50,7 @@
 #include "nsINode.h"
 #include "nsPIDOMWindow.h"
 #include "nsDOMPopStateEvent.h"
+#include "nsFrameLoader.h"
 
 #define NS_TARGET_CHAIN_FORCE_CONTENT_DISPATCH  (1 << 0)
 #define NS_TARGET_CHAIN_WANTS_WILL_HANDLE_EVENT (1 << 1)
@@ -472,6 +473,12 @@ nsEventDispatcher::Dispatch(nsISupports* aTarget,
                  NS_ERROR_ILLEGAL_VALUE);
   NS_ASSERTION(!aTargets || !aEvent->message, "Wrong parameters!");
 
+  // If we're dispatching an already created DOMEvent object, make
+  // sure it is initialized!
+  // If aTargets is non-null, the event isn't going to be dispatched.
+  NS_ENSURE_TRUE(aEvent->message || !aDOMEvent || aTargets,
+                 NS_ERROR_DOM_UNSPECIFIED_EVENT_TYPE_ERR);
+
 #ifdef NS_FUNCTION_TIMER
   const char* timer_event_name = nsDOMEvent::GetEventName(aEvent->message);
   NS_TIME_FUNCTION_MIN_FMT(20, "Dispatching '%s' event",
@@ -494,11 +501,22 @@ nsEventDispatcher::Dispatch(nsISupports* aTarget,
     if (!nsContentUtils::IsChromeDoc(doc)) {
       nsPIDOMWindow* win = doc ? doc->GetInnerWindow() : nsnull;
       // If we can't dispatch the event to chrome, do nothing.
-      NS_ENSURE_TRUE(win && win->GetChromeEventHandler(), NS_OK);
+      nsPIDOMEventTarget* piTarget = win ? win->GetChromeEventHandler() : nsnull;
+      NS_ENSURE_TRUE(piTarget, NS_OK);
+
+      nsCOMPtr<nsIFrameLoaderOwner> flo = do_QueryInterface(piTarget);
+      if (flo) {
+        nsRefPtr<nsFrameLoader> fl = flo->GetFrameLoader();
+        if (fl) {
+          nsPIDOMEventTarget* t = fl->GetTabChildGlobalAsEventTarget();
+          piTarget = t ? t : piTarget;
+        }
+      }
+      
       // Set the target to be the original dispatch target,
       aEvent->target = target;
-      // but use chrome event handler for event target chain.
-      target = do_QueryInterface(win->GetChromeEventHandler());
+      // but use chrome event handler or TabChildGlobal for event target chain.
+      target = piTarget;
     }
   }
 
@@ -759,6 +777,11 @@ nsEventDispatcher::CreateEvent(nsPresContext* aPresContext,
     case NS_TRANSITION_EVENT:
       return NS_NewDOMTransitionEvent(aDOMEvent, aPresContext,
                                       static_cast<nsTransitionEvent*>(aEvent));
+#ifdef MOZ_CSS_ANIMATIONS
+    case NS_ANIMATION_EVENT:
+      return NS_NewDOMAnimationEvent(aDOMEvent, aPresContext,
+                                     static_cast<nsAnimationEvent*>(aEvent));
+#endif
     }
 
     // For all other types of events, create a vanilla event object.
@@ -838,6 +861,10 @@ nsEventDispatcher::CreateEvent(nsPresContext* aPresContext,
   // is probably wrong!
   if (aEventType.LowerCaseEqualsLiteral("transitionevent"))
     return NS_NewDOMTransitionEvent(aDOMEvent, aPresContext, nsnull);
+#ifdef MOZ_CSS_ANIMATIONS
+  if (aEventType.LowerCaseEqualsLiteral("animationevent"))
+    return NS_NewDOMAnimationEvent(aDOMEvent, aPresContext, nsnull);
+#endif
   if (aEventType.LowerCaseEqualsLiteral("popstateevent"))
     return NS_NewDOMPopStateEvent(aDOMEvent, aPresContext, nsnull);
   if (aEventType.LowerCaseEqualsLiteral("mozaudioavailableevent"))
