@@ -20,7 +20,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Jesse Ruderman
+ * Contributor(s): Igor Bukanov
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -37,20 +37,21 @@
  * ***** END LICENSE BLOCK ***** */
 
 
-var BUGNUMBER = 355474;
-var summary = 'Iterating over XML with WAY_TOO_MUCH_GC';
-var actual = '';
-var expect = '';
+var BUGNUMBER = 354145;
+var summary = 'Immutable XML';
+var actual = 'No Crash';
+var expect = 'No Crash';
 
 printBugNumber(BUGNUMBER);
 START(summary);
 
-expect = '<a>text</a>';
-for each (var i in <><a>text</a></>)
-{
-    printStatus(i.toXMLString());
-    actual = i.toXMLString();
-}
+
+var a = <a><b/></a>
+try {
+  a.insertChildAfter(a.b[0], {toString: function() { throw 1; }});
+} catch (e) { }
+
+for each (var i in a) { }
 
 TEST(1, expect, actual);
 
