@@ -23,6 +23,8 @@
  *
  * Contributor(s):
  *   Igor Bukanov
+ *   Ethan Hugg
+ *   Milen Nankov
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -39,7 +41,7 @@
  * ***** END LICENSE BLOCK ***** */
 
 
-START("9.1.1.4 - XML [[DeleteByIndex]]");
+START("9.1.1.7 - XML [[DeepCopy]]");
 
 var actual = 'Test not implemented';
 var expect = 'PASS';
@@ -47,3 +49,4 @@ var expect = 'PASS';
 TEST(1, expect, actual);
 
 END();
+
