@@ -41,11 +41,50 @@
  * ***** END LICENSE BLOCK ***** */
 
 
-START("13.4.4.1 - XML Constructor");
+START("13.4.4.35 - setName");
 
-var actual = 'Test not implemented';
-var expect = 'PASS';
+TEST(1, true, XML.prototype.hasOwnProperty("setName"));
 
-TEST(1, expect, actual);
+x =
+<alpha>
+    <bravo>one</bravo>
+</alpha>;
+
+correct =
+<charlie>
+    <bravo>one</bravo>
+</charlie>;
+
+x.setName("charlie");
+
+TEST(2, correct, x);
+
+x =
+<ns:alpha xmlns:ns="http://foobar/">
+    <ns:bravo>one</ns:bravo>
+</ns:alpha>;
+
+correct =
+<charlie xmlns:ns="http://foobar/">
+    <ns:bravo>one</ns:bravo>
+</charlie>;
+
+x.setName("charlie");
+
+TEST(3, correct, x);
+
+x =
+<ns:alpha xmlns:ns="http://foobar/">
+    <ns:bravo>one</ns:bravo>
+</ns:alpha>;
+
+correct =
+<ns:charlie xmlns:ns="http://foobar/">
+    <ns:bravo>one</ns:bravo>
+</ns:charlie>;
+
+x.setName(new QName("http://foobar/", "charlie"));
+
+TEST(4, correct, x);
 
 END();
