@@ -40,48 +40,65 @@
  *
  * ***** END LICENSE BLOCK ***** */
 
-gTestfile = '13.5.4.2.js';
 
-START("13.5.4.2 - XMLList attribute()");
+START("13.5.4.13 - XMLList hasSimpleContent()");
 
-TEST(1, true, XMLList.prototype.hasOwnProperty("attribute"));
+TEST(1, true, XMLList.prototype.hasOwnProperty("hasSimpleContent"));
 
-// Test with list of size 0
-emps = new XMLList();
-TEST(2, "xml", typeof(emps.attribute("id")));
-TEST_XML(3, "", emps.attribute("id"));
+// One element should be same as XML case
+x =
+<>
+<alpha attr1="value1">
+    <bravo>one</bravo>
+    <charlie>
+        two
+        three
+        <echo>four</echo>
+    </charlie>
+    <delta />
+    <foxtrot attr2="value2">five</foxtrot>
+    <golf attr3="value3" />
+    <hotel>
+        six
+        seven
+    </hotel>
+    <india><juliet /></india>
+</alpha>;
+</>;
 
-// Test with list of size 1
-emps += <employee id="0"><name>Jim</name><age>25</age></employee>;
+TEST(2, false, x.hasSimpleContent());
+TEST(3, true, x.bravo.hasSimpleContent());
+TEST(4, false, x.charlie.hasSimpleContent());
+TEST(5, true, x.delta.hasSimpleContent());
+TEST(6, true, x.foxtrot.hasSimpleContent());
+TEST(7, true, x.golf.hasSimpleContent());
+TEST(8, true, x.hotel.hasSimpleContent());
+TEST(9, true, x.@attr1.hasSimpleContent());
+TEST(10, true, x.bravo.child(0).hasSimpleContent());
+TEST(11, false, x.india.hasSimpleContent());
 
-TEST(4, "xml", typeof(emps.attribute("id")));
-TEST_XML(5, 0, emps.attribute("id"));
+// More than one element is complex if one or more things in the list are elements.
+x =
+<>
+<alpha>one</alpha>
+<bravo>two</bravo>
+</>;
 
-// Test with list of size > 1
-emps += <employee id="1"><name>Joe</name><age>20</age></employee>;
-TEST(6, "xml", typeof(emps.attribute("id")));
+TEST(12, false, x.hasSimpleContent());
 
-correct = new XMLList();
-correct += new XML("0");
-correct += new XML("1");
-TEST(7, correct, emps.attribute("id"));
+x =
+<root>
+    one
+    <alpha>one</alpha>
+</root>;
 
-// Test one that doesn't exist - should return empty XMLList
-TEST(8, "xml", typeof(emps.attribute("foobar")));
+TEST(13, false, x.*.hasSimpleContent());
 
-// Test args of null and undefined
-try {
-  emps.attribute(null);
-  SHOULD_THROW(9);
-} catch (ex) {
-  TEST(9, "TypeError", ex.name);
-}
+x =
+<root attr1="value1" attr2="value2">
+    one
+</root>;
 
-try {
-  emps.attribute(undefined);
-  SHOULD_THROW(10);
-} catch (ex) {
-  TEST(10, "TypeError", ex.name);
-}
+TEST(14, true, x.@*.hasSimpleContent());
 
-END();
\ No newline at end of file
+END();
