@@ -38,28 +38,33 @@
 
 
 /**
-   File Name:          15.7.3.4-4.js
-   ECMA Section:       15.7.3.4 Number.NaN
+   File Name:          15.7.3.1-2.js
+   ECMA Section:       15.7.3.1 Number.prototype
    Description:        All value properties of the Number object should have
    the attributes [DontEnum, DontDelete, ReadOnly]
 
-   this test checks the DontEnum attribute of Number.NaN
+   this test checks the DontDelete attribute of Number.prototype
 
    Author:             christine@netscape.com
    Date:               16 september 1997
 */
 
-var SECTION = "15.7.3.4-4";
+
+var SECTION = "15.7.3.1-1";
 var VERSION = "ECMA_1";
 startTest();
-var TITLE   = "Number.NaN";
+var TITLE   = "Number.prototype";
 
-writeHeaderToLog( SECTION + " " + TITLE);
+writeHeaderToLog( SECTION +" "+ TITLE);
 
-new TestCase( SECTION,
-	      "var string = ''; for ( prop in Number ) { string += ( prop == 'NaN' ) ? prop : '' } string;",
-	      "",
-	      eval("var string = ''; for ( prop in Number ) { string += ( prop == 'NaN' ) ? prop : '' } string;")
-  );
+new TestCase(SECTION,
+	     "var NUM_PROT = Number.prototype; delete( Number.prototype ); NUM_PROT == Number.prototype",   
+	     true, 
+	     eval("var NUM_PROT = Number.prototype; delete( Number.prototype ); NUM_PROT == Number.prototype") );
+
+new TestCase(SECTION,
+	     "delete( Number.prototype )",         
+	     false,      
+	     eval("delete( Number.prototype )") );
 
 test();
