@@ -38,34 +38,64 @@
 
 
 /**
-   File Name:          12.6.2-3.js
-   ECMA Section:       12.6.2 The for Statement
+   File Name:          12.5-1.js
+   ECMA Section:       The if statement
+   Description:
 
-   1. first expression is not present.
-   2. second expression is present
-   3. third expression is present
+   The production IfStatement : if ( Expression ) Statement else Statement
+   is evaluated as follows:
 
+   1.Evaluate Expression.
+   2.Call GetValue(Result(1)).
+   3.Call ToBoolean(Result(2)).
+   4.If Result(3) is false, go to step 7.
+   5.Evaluate the first Statement.
+   6.Return Result(5).
+   7.Evaluate the second Statement.
+   8.Return Result(7).
 
    Author:             christine@netscape.com
-   Date:               15 september 1997
+   Date:               12 november 1997
 */
-var SECTION = "12.6.2-3";
+
+
+var SECTION = "12.5-1";
 var VERSION = "ECMA_1";
 startTest();
-var TITLE   = "The for statement";
+var TITLE   = "The if statement";
 
 writeHeaderToLog( SECTION + " "+ TITLE);
 
-new TestCase( SECTION, "for statement",  100,     testprogram() );
+
+new TestCase(   SECTION,
+		"var MYVAR; if ( true ) MYVAR='PASSED'; else MYVAR= 'FAILED';",
+		"PASSED",
+		eval("var MYVAR; if ( true ) MYVAR='PASSED'; else MYVAR= 'FAILED';") );
+
+new TestCase(  SECTION,
+	       "var MYVAR; if ( false ) MYVAR='FAILED'; else MYVAR= 'PASSED';",
+	       "PASSED",
+	       eval("var MYVAR; if ( false ) MYVAR='FAILED'; else MYVAR= 'PASSED';") );
+
+new TestCase(   SECTION,
+		"var MYVAR; if ( new Boolean(true) ) MYVAR='PASSED'; else MYVAR= 'FAILED';",
+		"PASSED",
+		eval("var MYVAR; if ( new Boolean(true) ) MYVAR='PASSED'; else MYVAR= 'FAILED';") );
+
+new TestCase(  SECTION,
+	       "var MYVAR; if ( new Boolean(false) ) MYVAR='PASSED'; else MYVAR= 'FAILED';",
+	       "PASSED",
+	       eval("var MYVAR; if ( new Boolean(false) ) MYVAR='PASSED'; else MYVAR= 'FAILED';") );
+
+new TestCase(   SECTION,
+		"var MYVAR; if ( 1 ) MYVAR='PASSED'; else MYVAR= 'FAILED';",
+		"PASSED",
+		eval("var MYVAR; if ( 1 ) MYVAR='PASSED'; else MYVAR= 'FAILED';") );
+
+new TestCase(  SECTION,
+	       "var MYVAR; if ( 0 ) MYVAR='FAILED'; else MYVAR= 'PASSED';",
+	       "PASSED",
+	       eval("var MYVAR; if ( 0 ) MYVAR='FAILED'; else MYVAR= 'PASSED';") );
 
 test();
 
-function testprogram() {
-  myVar = 0;
-
-  for ( ; myVar < 100 ; myVar++ ) {
-    continue;
-  }
-
-  return myVar;
-}
