@@ -19,7 +19,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2006
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Jesse Ruderman
+ * Contributor(s):
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,11 +36,12 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 349482;
-var summary = 'Decompiling try/catch in for..in should not crash';
+var BUGNUMBER = 360969;
+var summary = '2^17: local const';
 var actual = 'No Crash';
 var expect = 'No Crash';
 
+var global = this;
 
 //-----------------------------------------------------------------------------
 test();
@@ -52,10 +53,21 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
  
-  var f = function() { for(p in {})   try{}catch(e){}   };
-  print(f.toString());
+  var start = new Date();
+  var p;
+  var i;
+  var limit = 2 << 16;
+
+  for (var i = 0; i < limit; i++)
+  {
+    eval('const pv' + i + ';');
+  }
 
   reportCompare(expect, actual, summary);
 
+  var stop = new Date();
+
+  print('Elapsed time: ' + Math.floor((stop - start)/1000) + ' seconds');
+
   exitFunc ('test');
 }
