@@ -16,10 +16,10 @@
  *
  * The Initial Developer of the Original Code is
  * Mozilla Foundation.
- * Portions created by the Initial Developer are Copyright (C) 2006
+ * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): Jesse Ruderman
+ * Contributor(s): Gary Kwong
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,10 +36,10 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 355052;
-var summary = 'Do not crash with valueOf:gc and __iterator__';
-var actual = 'No Crash';
-var expect = 'No Crash';
+var BUGNUMBER = 472450;
+var summary = 'TM: Do not assert: StackBase(fp) + blockDepth == regs.sp';
+var actual = '';
+var expect = '';
 
 
 //-----------------------------------------------------------------------------
@@ -52,18 +52,15 @@ function test()
   printBugNumber(BUGNUMBER);
   printStatus (summary);
  
-  expect = /TypeError: NaN is not a function/;
-  actual = 'No Error';
-  try
-  {
-    ( {valueOf: gc} - [function(){}].__iterator__ )();
-  }
-  catch(ex)
-  {
-    actual = ex + '';
-  }
+  jit(true);
 
-  reportMatch(expect, actual, summary);
+  ({__proto__: #1=[#1#]})
+    for (var y = 0; y < 3; ++y) { for each (let z in ['', function(){}]) { let x =
+                                               1, c = []; } }
+
+  jit(false);
+
+  reportCompare(expect, actual, summary);
 
   exitFunc ('test');
 }
