@@ -37,7 +37,7 @@
 
 //-----------------------------------------------------------------------------
 var BUGNUMBER = 474771;
-var summary = 'TM: do not halt execution with gczeal, prototype mangling, for..in';
+var summary = 'gczeal, prototype mangling, for..in';
 var actual = '';
 var expect = '';
 
@@ -53,8 +53,6 @@ function test()
   printStatus (summary);
 
   expect = 'PASS';
-  jit(true);
-
   if (typeof gczeal == 'function')
   {
     gczeal(2);
@@ -62,10 +60,6 @@ function test()
 
   Object.prototype.q = 3;
   for each (let x in [6, 7]) { } print(actual = "PASS");
- 
-  jit(false);
-
-  delete Object.prototype.q;
 
   if (typeof gczeal == 'function')
   {
