@@ -19,7 +19,7 @@
  * Portions created by the Initial Developer are Copyright (C) 2008
  * the Initial Developer. All Rights Reserved.
  *
- * Contributor(s): nanto_vi (TOYAMA Nao)
+ * Contributor(s):
  *
  * Alternatively, the contents of this file may be used under the terms of
  * either the GNU General Public License Version 2 or later (the "GPL"), or
@@ -36,10 +36,10 @@
  * ***** END LICENSE BLOCK ***** */
 
 //-----------------------------------------------------------------------------
-var BUGNUMBER = 433279;
-var summary = 'Do not assert: pn != tc->parseContext->nodeList';
-var actual = 'No Crash';
-var expect = 'No Crash';
+var BUGNUMBER = 452498;
+var summary = 'TM: upvar2 regression tests';
+var actual = '';
+var expect = '';
 
 
 //-----------------------------------------------------------------------------
@@ -51,8 +51,29 @@ function test()
   enterFunc ('test');
   printBugNumber(BUGNUMBER);
   printStatus (summary);
- 
-  var { sin, PI } = Math; sin(PI / 2);
+
+// ------- Comment #131 From Gary Kwong [:nth10sd]
+
+// Does not require -j:
+// =====
+  try
+  {
+    eval('((__defineGetter__, function (x) { function x(){} }) for');
+  }
+  catch(ex)
+  {
+  }
+// Assertion failure: pn->pn_cookie == FREE_UPVAR_COOKIE, at ../jsparse.cpp:5698
+// =====
+  try
+  {
+    eval('( ""  ? 1.3 : x); *::*; x::x;');
+  }
+  catch(ex)
+  {
+  }
+// Assertion failure: pn != dn->dn_uses, at ../jsparse.cpp:1171
+// =====
 
   reportCompare(expect, actual, summary);
 
