@@ -48,7 +48,6 @@
 #include "nsIServiceManager.h"
 #include "nsILocalFile.h"
 #include "nsITimer.h"
-#include "nsInt64.h"
 #include "nsNetUtil.h"
 #include "nsAutoPtr.h"
 #include "nsWeakReference.h"
@@ -101,7 +100,7 @@ AppendToFile(nsILocalFile *lf, const char *data, PRUint32 len)
 
 // maxSize may be -1 if unknown
 static void
-MakeRangeSpec(const nsInt64 &size, const nsInt64 &maxSize, PRInt32 chunkSize,
+MakeRangeSpec(const PRInt64 &size, const PRInt64 &maxSize, PRInt32 chunkSize,
               PRBool fetchRemaining, nsCString &rangeSpec)
 {
   rangeSpec.AssignLiteral("bytes=");
@@ -111,8 +110,8 @@ MakeRangeSpec(const nsInt64 &size, const nsInt64 &maxSize, PRInt32 chunkSize,
   if (fetchRemaining)
     return;
 
-  nsInt64 end = size + nsInt64(chunkSize);
-  if (maxSize != nsInt64(-1) && end > maxSize)
+  PRInt64 end = size + PRInt64(chunkSize);
+  if (maxSize != PRInt64(-1) && end > maxSize)
     end = maxSize;
   end -= 1;
 
@@ -165,8 +164,8 @@ private:
   PRInt32                                  mChunkLen;
   PRInt32                                  mChunkSize;
   PRInt32                                  mInterval;
-  nsInt64                                  mTotalSize;
-  nsInt64                                  mCurrentSize;
+  PRInt64                                  mTotalSize;
+  PRInt64                                  mCurrentSize;
   PRUint32                                 mLoadFlags;
   PRInt32                                  mNonPartialCount;
   nsresult                                 mStatus;
@@ -197,7 +196,7 @@ nsIncrementalDownload::nsIncrementalDownload()
 nsresult
 nsIncrementalDownload::FlushChunk()
 {
-  NS_ASSERTION(mTotalSize != nsInt64(-1), "total size should be known");
+  NS_ASSERTION(mTotalSize != PRInt64(-1), "total size should be known");
 
   if (mChunkLen == 0)
     return NS_OK;
@@ -206,7 +205,7 @@ nsIncrementalDownload::FlushChunk()
   if (NS_FAILED(rv))
     return rv;
 
-  mCurrentSize += nsInt64(mChunkLen);
+  mCurrentSize += PRInt64(mChunkLen);
   mChunkLen = 0;
 
   return NS_OK;
@@ -285,7 +284,7 @@ nsIncrementalDownload::ProcessTimeout()
   if (NS_FAILED(rv))
     return rv;
 
-  NS_ASSERTION(mCurrentSize != nsInt64(-1),
+  NS_ASSERTION(mCurrentSize != PRInt64(-1),
       "we should know the current file size by now");
 
   rv = ClearRequestHeader(http);
@@ -294,7 +293,7 @@ nsIncrementalDownload::ProcessTimeout()
 
   // Don't bother making a range request if we are just going to fetch the
   // entire document.
-  if (mInterval || mCurrentSize != nsInt64(0)) {
+  if (mInterval || mCurrentSize != PRInt64(0)) {
     nsCAutoString range;
     MakeRangeSpec(mCurrentSize, mTotalSize, mChunkSize, mInterval == 0, range);
 
@@ -319,7 +318,7 @@ nsIncrementalDownload::ProcessTimeout()
 nsresult
 nsIncrementalDownload::ReadCurrentSize()
 {
-  nsInt64 size;
+  PRInt64 size;
   nsresult rv = mDest->GetFileSize((PRInt64 *) &size);
   if (rv == NS_ERROR_FILE_NOT_FOUND ||
       rv == NS_ERROR_FILE_TARGET_DOES_NOT_EXIST) {
@@ -553,7 +552,7 @@ nsIncrementalDownload::OnStartRequest(nsIRequest *request,
     // We may already have the entire file downloaded, in which case
     // our request for a range beyond the end of the file would have
     // been met with an error response code.
-    if (code == 416 && mTotalSize == nsInt64(-1)) {
+    if (code == 416 && mTotalSize == PRInt64(-1)) {
       mTotalSize = mCurrentSize;
       // Return an error code here to suppress OnDataAvailable.
       return NS_ERROR_DOWNLOAD_COMPLETE;
@@ -587,7 +586,7 @@ nsIncrementalDownload::OnStartRequest(nsIRequest *request,
   }
 
   // Do special processing after the first response.
-  if (mTotalSize == nsInt64(-1)) {
+  if (mTotalSize == PRInt64(-1)) {
     // Update knowledge of mFinalURI
     rv = http->GetURI(getter_AddRefs(mFinalURI));
     if (NS_FAILED(rv))
@@ -614,9 +613,9 @@ nsIncrementalDownload::OnStartRequest(nsIRequest *request,
       if (NS_FAILED(rv))
         return rv;
       rv = props->GetPropertyAsInt64(NS_CHANNEL_PROP_CONTENT_LENGTH,
-                                     &mTotalSize.mValue);
+                                     &mTotalSize);
       // We need to know the total size of the thing we're trying to download.
-      if (mTotalSize == nsInt64(-1)) {
+      if (mTotalSize == PRInt64(-1)) {
         NS_WARNING("server returned no content-length header!");
         return NS_ERROR_UNEXPECTED;
       }
@@ -633,13 +632,13 @@ nsIncrementalDownload::OnStartRequest(nsIRequest *request,
   }
 
   // Adjust mChunkSize accordingly if mCurrentSize is close to mTotalSize.
-  nsInt64 diff = mTotalSize - mCurrentSize;
-  if (diff <= nsInt64(0)) {
+  PRInt64 diff = mTotalSize - mCurrentSize;
+  if (diff <= PRInt64(0)) {
     NS_WARNING("about to set a bogus chunk size; giving up");
     return NS_ERROR_UNEXPECTED;
   }
 
-  if (diff < nsInt64(mChunkSize))
+  if (diff < PRInt64(mChunkSize))
     mChunkSize = PRUint32(diff);
 
   mChunk = new char[mChunkSize];
