@@ -37,11 +37,12 @@
 
 #include "nsThreadUtils.h"
 #include "nsSSLThread.h"
-#include "nsAutoLock.h"
 #include "nsNSSIOLayer.h"
 
 #include "ssl.h"
 
+using namespace mozilla;
+
 #ifdef PR_LOGGING
 extern PRLogModuleInfo* gPIPNSSLog;
 #endif
@@ -65,7 +66,7 @@ PRFileDesc *nsSSLThread::getRealSSLFD(nsNSSSocketInfo *si)
   if (!ssl_thread_singleton || !si || !ssl_thread_singleton->mThreadHandle)
     return nsnull;
 
-  nsAutoLock threadLock(ssl_thread_singleton->mMutex);
+  MutexAutoLock threadLock(ssl_thread_singleton->mMutex);
 
   if (si->mThreadData->mReplacedSSLFileDesc)
   {
@@ -134,10 +135,17 @@ PRInt32 nsSSLThread::requestRecvMsgPeek(nsNSSSocketInfo *si, void *buf, PRInt32
     return -1;
   }
 
+  // Socket is unusable - set error and return -1. See bug #480619.
+  if (si->isPK11LoggedOut() || si->isAlreadyShutDown())
+  {
+    PR_SetError(PR_SOCKET_SHUTDOWN_ERROR, 0);
+    return -1;
+  }
+
   PRFileDesc *realSSLFD;
 
   {
-    nsAutoLock threadLock(ssl_thread_singleton->mMutex);
+    MutexAutoLock threadLock(ssl_thread_singleton->mMutex);
 
     if (si == ssl_thread_singleton->mBusySocket)
     {
@@ -222,11 +230,18 @@ PRInt16 nsSSLThread::requestPoll(nsNSSSocketInfo *si, PRInt16 in_flags, PRInt16
 
   *out_flags = 0;
 
+  // Socket is unusable - set EXCEPT-flag and return. See bug #480619.
+  if (si->isPK11LoggedOut() || si->isAlreadyShutDown())
+  {
+    *out_flags |= PR_POLL_EXCEPT;
+    return in_flags;
+  }
+
   PRBool want_sleep_and_wakeup_on_any_socket_activity = PR_FALSE;
   PRBool handshake_timeout = PR_FALSE;
   
   {
-    nsAutoLock threadLock(ssl_thread_singleton->mMutex);
+    MutexAutoLock threadLock(ssl_thread_singleton->mMutex);
 
     if (ssl_thread_singleton->mBusySocket)
     {
@@ -381,7 +396,7 @@ PRStatus nsSSLThread::requestClose(nsNSSSocketInfo *si)
   nsCOMPtr<nsIRequest> requestToCancel;
 
   {
-    nsAutoLock threadLock(ssl_thread_singleton->mMutex);
+    MutexAutoLock threadLock(ssl_thread_singleton->mMutex);
 
     if (ssl_thread_singleton->mBusySocket == si) {
     
@@ -400,7 +415,7 @@ PRStatus nsSSLThread::requestClose(nsNSSSocketInfo *si)
       close_later = PR_TRUE;
       ssl_thread_singleton->mSocketScheduledToBeDestroyed = si;
 
-      PR_NotifyAllCondVar(ssl_thread_singleton->mCond);
+      ssl_thread_singleton->mCond.NotifyAll();
     }
   }
 
@@ -493,7 +508,7 @@ PRInt32 nsSSLThread::requestRead(nsNSSSocketInfo *si, void *buf, PRInt32 amount,
   PRFileDesc *blockingFD = nsnull;
 
   {
-    nsAutoLock threadLock(ssl_thread_singleton->mMutex);
+    MutexAutoLock threadLock(ssl_thread_singleton->mMutex);
 
     if (ssl_thread_singleton->mExitRequested) {
       PR_SetError(PR_UNKNOWN_ERROR, 0);
@@ -681,7 +696,7 @@ PRInt32 nsSSLThread::requestRead(nsNSSSocketInfo *si, void *buf, PRInt32 amount,
   // Finally, we return the data obtained on the SSL thread back to our caller.
 
   {
-    nsAutoLock threadLock(ssl_thread_singleton->mMutex);
+    MutexAutoLock threadLock(ssl_thread_singleton->mMutex);
 
     if (nsSSLIOLayerHelpers::mSharedPollableEvent)
     {
@@ -698,7 +713,7 @@ PRInt32 nsSSLThread::requestRead(nsNSSSocketInfo *si, void *buf, PRInt32 amount,
     ssl_thread_singleton->mBusySocket = si;
 
     // notify the thread
-    PR_NotifyAllCondVar(ssl_thread_singleton->mCond);
+    ssl_thread_singleton->mCond.NotifyAll();
   }
 
   PORT_SetError(PR_WOULD_BLOCK_ERROR);
@@ -720,7 +735,7 @@ PRInt32 nsSSLThread::requestWrite(nsNSSSocketInfo *si, const void *buf, PRInt32
   PRFileDesc *blockingFD = nsnull;
   
   {
-    nsAutoLock threadLock(ssl_thread_singleton->mMutex);
+    MutexAutoLock threadLock(ssl_thread_singleton->mMutex);
     
     if (ssl_thread_singleton->mExitRequested) {
       PR_SetError(PR_UNKNOWN_ERROR, 0);
@@ -805,6 +820,8 @@ PRInt32 nsSSLThread::requestWrite(nsNSSSocketInfo *si, const void *buf, PRInt32
           return si->mThreadData->mSSLResultRemainingBytes;
         }
 
+        nsSSLIOLayerHelpers::rememberTolerantSite(si->mFd, si);
+
         PRInt32 return_amount = NS_MIN(amount, si->mThreadData->mSSLResultRemainingBytes);
 
         si->mThreadData->mSSLResultRemainingBytes -= return_amount;
@@ -876,7 +893,7 @@ PRInt32 nsSSLThread::requestWrite(nsNSSSocketInfo *si, const void *buf, PRInt32
   si->mThreadData->mSSLState = nsSSLSocketThreadData::ssl_pending_write;
 
   {
-    nsAutoLock threadLock(ssl_thread_singleton->mMutex);
+    MutexAutoLock threadLock(ssl_thread_singleton->mMutex);
 
     if (nsSSLIOLayerHelpers::mSharedPollableEvent)
     {
@@ -892,7 +909,7 @@ PRInt32 nsSSLThread::requestWrite(nsNSSSocketInfo *si, const void *buf, PRInt32
     nsSSLIOLayerHelpers::mSocketOwningPollableEvent = si;
     ssl_thread_singleton->mBusySocket = si;
 
-    PR_NotifyAllCondVar(ssl_thread_singleton->mCond);
+    ssl_thread_singleton->mCond.NotifyAll();
   }
 
   PORT_SetError(PR_WOULD_BLOCK_ERROR);
@@ -920,7 +937,7 @@ void nsSSLThread::Run(void)
       // In this scope we need mutex protection,
       // as we find out what needs to be done.
       
-      nsAutoLock threadLock(ssl_thread_singleton->mMutex);
+      MutexAutoLock threadLock(ssl_thread_singleton->mMutex);
       
       if (mSocketScheduledToBeDestroyed)
       {
@@ -961,7 +978,7 @@ void nsSSLThread::Run(void)
         {
           // no work to do ? let's wait a moment
 
-          PR_WaitCondVar(mCond, PR_INTERVAL_NO_TIMEOUT);
+          mCond.Wait();
         }
         
       } while (!pending_work && !mExitRequested && !mSocketScheduledToBeDestroyed);
@@ -1079,7 +1096,7 @@ void nsSSLThread::Run(void)
     PRBool needToSetPollableEvent = PR_FALSE;
 
     {
-      nsAutoLock threadLock(ssl_thread_singleton->mMutex);
+      MutexAutoLock threadLock(ssl_thread_singleton->mMutex);
       
       mBusySocket->mThreadData->mSSLState = busy_socket_ssl_state;
       
@@ -1103,7 +1120,7 @@ void nsSSLThread::Run(void)
   }
 
   {
-    nsAutoLock threadLock(ssl_thread_singleton->mMutex);
+    MutexAutoLock threadLock(ssl_thread_singleton->mMutex);
     if (mBusySocket)
     {
       restoreOriginalSocket_locked(mBusySocket);
