@@ -36,7 +36,7 @@
  * the terms of any one of the MPL, the GPL or the LGPL.
  *
  * ***** END LICENSE BLOCK ***** */
-/* $Id: nssinit.c,v 1.99 2009/07/23 01:56:40 nelson%bolyard.com Exp $ */
+/* $Id: nssinit.c,v 1.106 2010/04/03 20:06:00 nelson%bolyard.com Exp $ */
 
 #include <ctype.h>
 #include <string.h>
@@ -46,8 +46,6 @@
 #include "prmem.h"
 #include "cert.h"
 #include "key.h"
-#include "ssl.h"
-#include "sslproto.h"
 #include "secmod.h"
 #include "secoid.h"
 #include "nss.h"
@@ -128,6 +126,89 @@ nss_makeFlags(PRBool readOnly, PRBool noCertDB,
     return flags;
 }
 
+
+/*
+ * build config string from individual internationalized strings
+ */
+char *
+nss_MkConfigString(const char *man, const char *libdesc, const char *tokdesc,
+	const char *ptokdesc, const char *slotdesc, const char *pslotdesc, 
+	const char *fslotdesc, const char *fpslotdesc, int minPwd)
+{
+    char *strings = NULL;
+    char *newStrings;
+
+    /* make sure the internationalization was done correctly... */
+    strings = PR_smprintf("");
+    if (strings == NULL) return NULL;
+
+    if (man) {
+        newStrings = PR_smprintf("%s manufacturerID='%s'",strings,man);
+	PR_smprintf_free(strings);
+	strings = newStrings;
+    }
+    if (strings == NULL) return NULL;
+
+    if (libdesc) {
+        newStrings = PR_smprintf("%s libraryDescription='%s'",strings,libdesc);
+	PR_smprintf_free(strings);
+	strings = newStrings;
+    }
+    if (strings == NULL) return NULL;
+
+    if (tokdesc) {
+        newStrings = PR_smprintf("%s cryptoTokenDescription='%s'",strings,
+								tokdesc);
+	PR_smprintf_free(strings);
+	strings = newStrings;
+    }
+    if (strings == NULL) return NULL;
+
+    if (ptokdesc) {
+        newStrings = PR_smprintf("%s dbTokenDescription='%s'",strings,ptokdesc);
+	PR_smprintf_free(strings);
+	strings = newStrings;
+    }
+    if (strings == NULL) return NULL;
+
+    if (slotdesc) {
+        newStrings = PR_smprintf("%s cryptoSlotDescription='%s'",strings,
+								slotdesc);
+	PR_smprintf_free(strings);
+	strings = newStrings;
+    }
+    if (strings == NULL) return NULL;
+
+    if (pslotdesc) {
+        newStrings = PR_smprintf("%s dbSlotDescription='%s'",strings,pslotdesc);
+	PR_smprintf_free(strings);
+	strings = newStrings;
+    }
+    if (strings == NULL) return NULL;
+
+    if (fslotdesc) {
+        newStrings = PR_smprintf("%s FIPSSlotDescription='%s'",
+							strings,fslotdesc);
+	PR_smprintf_free(strings);
+	strings = newStrings;
+    }
+    if (strings == NULL) return NULL;
+
+    if (fpslotdesc) {
+        newStrings = PR_smprintf("%s FIPSTokenDescription='%s'",
+							strings,fpslotdesc);
+	PR_smprintf_free(strings);
+	strings = newStrings;
+    }
+    if (strings == NULL) return NULL;
+
+    newStrings = PR_smprintf("%s minPS=%d", strings, minPwd);
+    PR_smprintf_free(strings);
+    strings = newStrings;
+
+    return(strings);
+}
+
 /*
  * statics to remember the PK11_ConfigurePKCS11()
  * info.
@@ -141,85 +222,25 @@ static PRBool pk11_password_required = PR_FALSE;
  * the PKCS #11 internal token.
  */
 void
-PK11_ConfigurePKCS11(const char *man, const char *libdes, const char *tokdes,
-	const char *ptokdes, const char *slotdes, const char *pslotdes, 
-	const char *fslotdes, const char *fpslotdes, int minPwd, int pwRequired)
+PK11_ConfigurePKCS11(const char *man, const char *libdesc, const char *tokdesc,
+	const char *ptokdesc, const char *slotdesc, const char *pslotdesc, 
+	const char *fslotdesc, const char *fpslotdesc, int minPwd, 
+	int pwRequired)
 {
-   char *strings = NULL;
-   char *newStrings;
+    char * strings;
 
-   /* make sure the internationalization was done correctly... */
-   strings = PR_smprintf("");
-   if (strings == NULL) return;
-
-    if (man) {
-        newStrings = PR_smprintf("%s manufacturerID='%s'",strings,man);
-	PR_smprintf_free(strings);
-	strings = newStrings;
+    strings = nss_MkConfigString(man,libdesc,tokdesc,ptokdesc,slotdesc,
+	pslotdesc,fslotdesc,fpslotdesc,minPwd);
+    if (strings == NULL) {
+	return;
     }
-   if (strings == NULL) return;
 
-    if (libdes) {
-        newStrings = PR_smprintf("%s libraryDescription='%s'",strings,libdes);
-	PR_smprintf_free(strings);
-	strings = newStrings;
+    if (libdesc) {
 	if (pk11_config_name != NULL) {
 	    PORT_Free(pk11_config_name);
 	}
-	pk11_config_name = PORT_Strdup(libdes);
+	pk11_config_name = PORT_Strdup(libdesc);
     }
-   if (strings == NULL) return;
-
-    if (tokdes) {
-        newStrings = PR_smprintf("%s cryptoTokenDescription='%s'",strings,
-								tokdes);
-	PR_smprintf_free(strings);
-	strings = newStrings;
-    }
-   if (strings == NULL) return;
-
-    if (ptokdes) {
-        newStrings = PR_smprintf("%s dbTokenDescription='%s'",strings,ptokdes);
-	PR_smprintf_free(strings);
-	strings = newStrings;
-    }
-   if (strings == NULL) return;
-
-    if (slotdes) {
-        newStrings = PR_smprintf("%s cryptoSlotDescription='%s'",strings,
-								slotdes);
-	PR_smprintf_free(strings);
-	strings = newStrings;
-    }
-   if (strings == NULL) return;
-
-    if (pslotdes) {
-        newStrings = PR_smprintf("%s dbSlotDescription='%s'",strings,pslotdes);
-	PR_smprintf_free(strings);
-	strings = newStrings;
-    }
-   if (strings == NULL) return;
-
-    if (fslotdes) {
-        newStrings = PR_smprintf("%s FIPSSlotDescription='%s'",
-							strings,fslotdes);
-	PR_smprintf_free(strings);
-	strings = newStrings;
-    }
-   if (strings == NULL) return;
-
-    if (fpslotdes) {
-        newStrings = PR_smprintf("%s FIPSTokenDescription='%s'",
-							strings,fpslotdes);
-	PR_smprintf_free(strings);
-	strings = newStrings;
-    }
-   if (strings == NULL) return;
-
-    newStrings = PR_smprintf("%s minPS=%d", strings, minPwd);
-    PR_smprintf_free(strings);
-    strings = newStrings;
-   if (strings == NULL) return;
 
     if (pk11_config_strings != NULL) {
 	PR_smprintf_free(pk11_config_strings);
@@ -242,56 +263,6 @@ void PK11_UnconfigurePKCS11(void)
     }
 }
 
-static char *
-nss_addEscape(const char *string, char quote)
-{
-    char *newString = 0;
-    int escapes = 0, size = 0;
-    const char *src;
-    char *dest;
-
-    for (src=string; *src ; src++) {
-	if ((*src == quote) || (*src == '\\')) escapes++;
-	size++;
-    }
-
-    newString = PORT_ZAlloc(escapes+size+1); 
-    if (newString == NULL) {
-	return NULL;
-    }
-
-    for (src=string, dest=newString; *src; src++,dest++) {
-	if ((*src == '\\') || (*src == quote)) {
-	    *dest++ = '\\';
-	}
-	*dest = *src;
-    }
-
-    return newString;
-}
-
-static char *
-nss_doubleEscape(const char *string)
-{
-    char *round1 = NULL;
-    char *retValue = NULL;
-    if (string == NULL) {
-	goto done;
-    }
-    round1 = nss_addEscape(string,'\'');
-    if (round1) {
-	retValue = nss_addEscape(round1,'"');
-	PORT_Free(round1);
-    }
-
-done:
-    if (retValue == NULL) {
-	retValue = PORT_Strdup("");
-    }
-    return retValue;
-}
-
-
 /*
  * The following code is an attempt to automagically find the external root
  * module.
@@ -389,46 +360,25 @@ nss_FindExternalRoot(const char *dbpath, const char* secmodprefix)
 }
 
 /*
- * OK there are now lots of options here, lets go through them all:
+ * see nss_Init for definitions of the various options.
  *
- * configdir - base directory where all the cert, key, and module datbases live.
- * certPrefix - prefix added to the beginning of the cert database example: "
- * 			"https-server1-"
- * keyPrefix - prefix added to the beginning of the key database example: "
- * 			"https-server1-"
- * secmodName - name of the security module database (usually "secmod.db").
- * readOnly - Boolean: true if the databases are to be opened read only.
- * nocertdb - Don't open the cert DB and key DB's, just initialize the 
- *			Volatile certdb.
- * nomoddb - Don't open the security module DB, just initialize the 
- *			PKCS #11 module.
- * forceOpen - Continue to force initializations even if the databases cannot
- * 			be opened.
+ * this function builds a moduleSpec string from the options and previously
+ * set statics (from PKCS11_Configure, for instance), and uses it to kick off
+ * the loading of the various PKCS #11 modules.
  */
-
-static PRBool nss_IsInitted = PR_FALSE;
-static void* plContext = NULL;
-
-static SECStatus nss_InitShutdownList(void);
-
-#ifdef DEBUG
-static CERTCertificate dummyCert;
-#endif
-
 static SECStatus
-nss_Init(const char *configdir, const char *certPrefix, const char *keyPrefix,
-		 const char *secmodName, const char *updateDir, 
-		 const char *updCertPrefix, const char *updKeyPrefix,
-		 const char *updateID, const char *updateName,
-			PRBool readOnly, PRBool noCertDB, 
-			PRBool noModDB, PRBool forceOpen, PRBool noRootInit,
-			PRBool optimizeSpace, PRBool noSingleThreadedModules,
-			PRBool allowAlreadyInitializedModules,
-			PRBool dontFinalizeModules)
+nss_InitModules(const char *configdir, const char *certPrefix, 
+		const char *keyPrefix, const char *secmodName, 
+		const char *updateDir, const char *updCertPrefix, 
+		const char *updKeyPrefix, const char *updateID, 
+		const char *updateName, char *configName, char *configStrings,
+		PRBool pwRequired, PRBool readOnly, PRBool noCertDB,
+		PRBool noModDB, PRBool forceOpen, PRBool optimizeSpace,
+		PRBool isContextInit)
 {
+    SECStatus rv = SECFailure;
     char *moduleSpec = NULL;
     char *flags = NULL;
-    SECStatus rv = SECFailure;
     char *lconfigdir = NULL;
     char *lcertPrefix = NULL;
     char *lkeyPrefix = NULL;
@@ -438,88 +388,62 @@ nss_Init(const char *configdir, const char *certPrefix, const char *keyPrefix,
     char *lupdKeyPrefix = NULL;
     char *lupdateID = NULL;
     char *lupdateName = NULL;
-    PKIX_UInt32 actualMinorVersion = 0;
-    PKIX_Error *pkixError = NULL;;
-
-    if (nss_IsInitted) {
-	return SECSuccess;
-    }
-
-    /* New option bits must not change the size of CERTCertificate. */
-    PORT_Assert(sizeof(dummyCert.options) == sizeof(void *));
-
-    if (SECSuccess != cert_InitLocks()) {
-        return SECFailure;
-    }
-
-    if (SECSuccess != InitCRLCache()) {
-        return SECFailure;
-    }
-    
-    if (SECSuccess != OCSP_InitGlobal()) {
-        return SECFailure;
-    }
 
     flags = nss_makeFlags(readOnly,noCertDB,noModDB,forceOpen,
-					pk11_password_required, optimizeSpace);
+					pwRequired, optimizeSpace);
     if (flags == NULL) return rv;
 
     /*
      * configdir is double nested, and Windows uses the same character
      * for file seps as we use for escapes! (sigh).
      */
-    lconfigdir = nss_doubleEscape(configdir);
+    lconfigdir = secmod_DoubleEscape(configdir, '\'', '\"');
     if (lconfigdir == NULL) {
 	goto loser;
     }
-    lcertPrefix = nss_doubleEscape(certPrefix);
+    lcertPrefix = secmod_DoubleEscape(certPrefix, '\'', '\"');
     if (lcertPrefix == NULL) {
 	goto loser;
     }
-    lkeyPrefix = nss_doubleEscape(keyPrefix);
+    lkeyPrefix = secmod_DoubleEscape(keyPrefix, '\'', '\"');
     if (lkeyPrefix == NULL) {
 	goto loser;
     }
-    lsecmodName = nss_doubleEscape(secmodName);
+    lsecmodName = secmod_DoubleEscape(secmodName, '\'', '\"');
     if (lsecmodName == NULL) {
 	goto loser;
     }
-    lupdateDir = nss_doubleEscape(updateDir);
+    lupdateDir = secmod_DoubleEscape(updateDir, '\'', '\"');
     if (lupdateDir == NULL) {
 	goto loser;
     }
-    lupdCertPrefix = nss_doubleEscape(updCertPrefix);
+    lupdCertPrefix = secmod_DoubleEscape(updCertPrefix, '\'', '\"');
     if (lupdCertPrefix == NULL) {
 	goto loser;
     }
-    lupdKeyPrefix = nss_doubleEscape(updKeyPrefix);
+    lupdKeyPrefix = secmod_DoubleEscape(updKeyPrefix, '\'', '\"');
     if (lupdKeyPrefix == NULL) {
 	goto loser;
     }
-    lupdateID = nss_doubleEscape(updateID);
+    lupdateID = secmod_DoubleEscape(updateID, '\'', '\"');
     if (lupdateID == NULL) {
 	goto loser;
     }
-    lupdateName = nss_doubleEscape(updateName);
+    lupdateName = secmod_DoubleEscape(updateName, '\'', '\"');
     if (lupdateName == NULL) {
 	goto loser;
     }
-    if (noSingleThreadedModules || allowAlreadyInitializedModules ||
-        dontFinalizeModules) {
-        pk11_setGlobalOptions(noSingleThreadedModules,
-                              allowAlreadyInitializedModules,
-                              dontFinalizeModules);
-    }
 
     moduleSpec = PR_smprintf(
      "name=\"%s\" parameters=\"configdir='%s' certPrefix='%s' keyPrefix='%s' "
      "secmod='%s' flags=%s updatedir='%s' updateCertPrefix='%s' "
      "updateKeyPrefix='%s' updateid='%s' updateTokenDescription='%s' %s\" "
-     "NSS=\"flags=internal,moduleDB,moduleDBOnly,critical\"",
-		pk11_config_name ? pk11_config_name : NSS_DEFAULT_MOD_NAME,
+     "NSS=\"flags=internal,moduleDB,moduleDBOnly,critical%s\"",
+		configName ? configName : NSS_DEFAULT_MOD_NAME,
 		lconfigdir,lcertPrefix,lkeyPrefix,lsecmodName,flags,
 		lupdateDir, lupdCertPrefix, lupdKeyPrefix, lupdateID, 
-		lupdateName, pk11_config_strings ? pk11_config_strings : "");
+		lupdateName, configStrings ? configStrings : "",
+		isContextInit ? "" : ",defaultModDB,internalKeySlot");
 
 loser:
     PORT_Free(flags);
@@ -541,65 +465,262 @@ loser:
 	    SECMOD_DestroyModule(module);
 	}
     }
+    return rv;
+}
 
-    if (rv == SECSuccess) {
+/*
+ * OK there are now lots of options here, lets go through them all:
+ *
+ * configdir - base directory where all the cert, key, and module datbases live.
+ * certPrefix - prefix added to the beginning of the cert database example: "
+ * 			"https-server1-"
+ * keyPrefix - prefix added to the beginning of the key database example: "
+ * 			"https-server1-"
+ * secmodName - name of the security module database (usually "secmod.db").
+ * updateDir - used in initMerge, old directory to update from.
+ * updateID - used in initMerge, unique ID to represent the updated directory.
+ * updateName - used in initMerge, token name when updating.
+ * initContextPtr -  used in initContext, pointer to return a unique context
+ *            value.
+ * readOnly - Boolean: true if the databases are to be opened read only.
+ * nocertdb - Don't open the cert DB and key DB's, just initialize the 
+ *			Volatile certdb.
+ * nomoddb - Don't open the security module DB, just initialize the 
+ *			PKCS #11 module.
+ * forceOpen - Continue to force initializations even if the databases cannot
+ * 			be opened.
+ * noRootInit - don't try to automatically load the root cert store if one is
+ *           not found.
+ * optimizeSpace - tell NSS to use fewer hash table buckets.
+ *
+ * The next three options are used in an attempt to share PKCS #11 modules
+ * with other loaded, running libraries. PKCS #11 was not designed with this
+ * sort of sharing in mind, so use of these options may lead to questionable
+ * results. These options are may be incompatible with NSS_LoadContext() calls.
+ *
+ * noSingleThreadedModules - don't load modules that are not thread safe (many
+ *           smart card tokens will not work).
+ * allowAlreadyInitializedModules - if a module has already been loaded and
+ *           initialize try to use it.
+ * don'tFinalizeModules -  dont shutdown modules we may have loaded.
+ */
+
+static PRBool          nssIsInitted = PR_FALSE;
+static NSSInitContext *nssInitContextList = NULL;
+static void*           plContext = NULL;
+
+struct NSSInitContextStr {
+    NSSInitContext *next;
+    PRUint32 magic;
+};
+
+#define NSS_INIT_MAGIC 0x1413A91C
+static SECStatus nss_InitShutdownList(void);
+
+#ifdef DEBUG
+static CERTCertificate dummyCert;
+#endif
+
+static SECStatus
+nss_Init(const char *configdir, const char *certPrefix, const char *keyPrefix,
+		 const char *secmodName, const char *updateDir, 
+		 const char *updCertPrefix, const char *updKeyPrefix,
+		 const char *updateID, const char *updateName,
+		 NSSInitContext ** initContextPtr,
+		 NSSInitParameters *initParams,
+		 PRBool readOnly, PRBool noCertDB, 
+		 PRBool noModDB, PRBool forceOpen, PRBool noRootInit,
+		 PRBool optimizeSpace, PRBool noSingleThreadedModules,
+		 PRBool allowAlreadyInitializedModules,
+		 PRBool dontFinalizeModules)
+{
+    SECStatus rv = SECFailure;
+    PKIX_UInt32 actualMinorVersion = 0;
+    PKIX_Error *pkixError = NULL;
+    PRBool isReallyInitted;
+    char *configStrings = NULL;
+    char *configName = NULL;
+    PRBool passwordRequired = PR_FALSE;
+
+    /* if we are trying to init with a traditional NSS_Init call, maintain
+     * the traditional idempotent behavior. */
+    if (!initContextPtr && nssIsInitted) {
+	return SECSuccess;
+    }
+
+    /* this tells us whether or not some library has already initialized us.
+     * if so, we don't want to double call some of the basic initialization
+     * functions */
+    isReallyInitted = NSS_IsInitialized();
+
+    if (!isReallyInitted) {
+	/* New option bits must not change the size of CERTCertificate. */
+	PORT_Assert(sizeof(dummyCert.options) == sizeof(void *));
+
+	if (SECSuccess != cert_InitLocks()) {
+            return SECFailure;
+	}
+
+	if (SECSuccess != InitCRLCache()) {
+            return SECFailure;
+	}
+    
+	if (SECSuccess != OCSP_InitGlobal()) {
+            return SECFailure;
+	}
+    }
+
+    if (noSingleThreadedModules || allowAlreadyInitializedModules ||
+        dontFinalizeModules) {
+        pk11_setGlobalOptions(noSingleThreadedModules,
+                              allowAlreadyInitializedModules,
+                              dontFinalizeModules);
+    }
+
+    if (initContextPtr) {
+	*initContextPtr = PORT_ZNew(NSSInitContext);
+	if (*initContextPtr == NULL) {
+	    goto loser;
+	}
+	/*
+	 * For traditional NSS_Init, we used the PK11_Configure() call to set
+	 * globals. with InitContext, we pass those strings in as parameters.
+	 *
+	 * This allows old NSS_Init calls to work as before, while at the same
+	 * time new calls and old calls will not interfere with each other.
+	 */
+        if (initParams) {
+	    if (initParams->length < sizeof(NSSInitParameters)) {
+		PORT_SetError(SEC_ERROR_INVALID_ARGS);
+		goto loser;
+	    }
+	    configStrings = nss_MkConfigString(initParams->manufactureID,
+		initParams->libraryDescription,
+		initParams->cryptoTokenDescription,
+		initParams->dbTokenDescription,
+		initParams->cryptoSlotDescription,
+		initParams->dbSlotDescription,
+		initParams->FIPSSlotDescription,
+		initParams->FIPSTokenDescription,
+		initParams->minPWLen);
+	    if (configStrings == NULL) {
+		PORT_SetError(SEC_ERROR_NO_MEMORY);
+		goto loser;
+	    }
+	    configName = initParams->libraryDescription;
+	    passwordRequired = initParams->passwordRequired;
+	}
+    } else {
+	configStrings = pk11_config_strings;
+	configName = pk11_config_name;
+	passwordRequired = pk11_password_required;
+    }
+
+    /* we always try to initialize the modules */
+    rv = nss_InitModules(configdir, certPrefix, keyPrefix, secmodName, 
+		updateDir, updCertPrefix, updKeyPrefix, updateID, 
+		updateName, configName, configStrings, passwordRequired,
+		readOnly, noCertDB, noModDB, forceOpen, optimizeSpace, 
+		(initContextPtr != NULL));
+
+    if (rv != SECSuccess) {
+	goto loser;
+    }
+
+
+    /* finish up initialization */
+    if (!isReallyInitted) {
 	if (SECOID_Init() != SECSuccess) {
-	    return SECFailure;
+	    goto loser;
 	}
 	if (STAN_LoadDefaultNSS3TrustDomain() != PR_SUCCESS) {
-	    return SECFailure;
+	    goto loser;
 	}
 	if (nss_InitShutdownList() != SECSuccess) {
-	    return SECFailure;
+	    goto loser;
 	}
 	CERT_SetDefaultCertDB((CERTCertDBHandle *)
 				STAN_GetDefaultTrustDomain());
 	if ((!noModDB) && (!noCertDB) && (!noRootInit)) {
 	    if (!SECMOD_HasRootCerts()) {
 		const char *dbpath = configdir;
+		/* handle supported database modifiers */
 		if (strncmp(dbpath, "sql:", 4) == 0) {
 		    dbpath += 4;
+		} else if(strncmp(dbpath, "dbm:", 4) == 0) {
+		    dbpath += 4;
+		} else if(strncmp(dbpath, "extern:", 7) == 0) {
+		    dbpath += 7;
+		} else if(strncmp(dbpath, "rdb:", 4) == 0) {
+		    /* if rdb: is specified, the configdir isn't really a 
+		     * path. Skip it */
+		    dbpath = NULL;
+		}
+		if (dbpath) {
+		    nss_FindExternalRoot(dbpath, secmodName);
 		}
-		nss_FindExternalRoot(dbpath, secmodName);
 	    }
 	}
+
 	pk11sdr_Init();
 	cert_CreateSubjectKeyIDHashTable();
-	nss_IsInitted = PR_TRUE;
-    }
 
-    if (SECSuccess == rv) {
 	pkixError = PKIX_Initialize
 	    (PKIX_FALSE, PKIX_MAJOR_VERSION, PKIX_MINOR_VERSION,
 	    PKIX_MINOR_VERSION, &actualMinorVersion, &plContext);
 
 	if (pkixError != NULL) {
-	    rv = SECFailure;
+	    goto loser;
 	} else {
             char *ev = getenv("NSS_ENABLE_PKIX_VERIFY");
             if (ev && ev[0]) {
                 CERT_SetUsePKIXForValidation(PR_TRUE);
             }
         }
+
+
     }
 
-    return rv;
+    /*
+     * Now mark the appropriate init state. If initContextPtr was passed
+     * in, then return the new context pointer and add it to the
+     * nssInitContextList. Otherwise set the global nss_isInitted flag
+     */
+    if (!initContextPtr) {
+	nssIsInitted = PR_TRUE;
+    } else {
+	(*initContextPtr)->magic = NSS_INIT_MAGIC;
+	(*initContextPtr)->next = nssInitContextList;
+	nssInitContextList = (*initContextPtr);
+    }
+
+    return SECSuccess;
+
+loser:
+    if (initContextPtr && *initContextPtr) {
+	PORT_Free(*initContextPtr);
+	*initContextPtr = NULL;
+	if (configStrings) {
+	   PR_smprintf_free(configStrings);
+	}
+    }
+    return SECFailure;
 }
 
 
 SECStatus
 NSS_Init(const char *configdir)
 {
-    return nss_Init(configdir, "", "", SECMOD_DB, "", "", "", "", "",
-		PR_TRUE, PR_FALSE, PR_FALSE, PR_FALSE, PR_FALSE, 
+    return nss_Init(configdir, "", "", SECMOD_DB, "", "", "", "", "", NULL,
+		NULL, PR_TRUE, PR_FALSE, PR_FALSE, PR_FALSE, PR_FALSE, 
 		PR_TRUE, PR_FALSE, PR_FALSE, PR_FALSE);
 }
 
 SECStatus
 NSS_InitReadWrite(const char *configdir)
 {
-    return nss_Init(configdir, "", "", SECMOD_DB, "", "", "", "", "",
-		PR_FALSE, PR_FALSE, PR_FALSE, PR_FALSE, PR_FALSE, 
+    return nss_Init(configdir, "", "", SECMOD_DB, "", "", "", "", "", NULL,
+		NULL, PR_FALSE, PR_FALSE, PR_FALSE, PR_FALSE, PR_FALSE, 
 		PR_TRUE, PR_FALSE, PR_FALSE, PR_FALSE);
 }
 
@@ -656,7 +777,7 @@ NSS_Initialize(const char *configdir, const char *certPrefix,
 	const char *keyPrefix, const char *secmodName, PRUint32 flags)
 {
     return nss_Init(configdir, certPrefix, keyPrefix, secmodName,
-	"", "", "", "", "",
+	"", "", "", "", "", NULL, NULL,
 	((flags & NSS_INIT_READONLY) == NSS_INIT_READONLY),
 	((flags & NSS_INIT_NOCERTDB) == NSS_INIT_NOCERTDB),
 	((flags & NSS_INIT_NOMODDB) == NSS_INIT_NOMODDB),
@@ -668,6 +789,27 @@ NSS_Initialize(const char *configdir, const char *certPrefix,
         ((flags & NSS_INIT_NOPK11FINALIZE) == NSS_INIT_NOPK11FINALIZE));
 }
 
+NSSInitContext *
+NSS_InitContext(const char *configdir, const char *certPrefix, 
+	const char *keyPrefix, const char *secmodName, 
+	NSSInitParameters *initParams, PRUint32 flags)
+{
+    SECStatus rv;
+    NSSInitContext *context;
+
+    rv = nss_Init(configdir, certPrefix, keyPrefix, secmodName,
+	"", "", "", "", "", &context, initParams,
+	((flags & NSS_INIT_READONLY) == NSS_INIT_READONLY),
+	((flags & NSS_INIT_NOCERTDB) == NSS_INIT_NOCERTDB),
+	((flags & NSS_INIT_NOMODDB) == NSS_INIT_NOMODDB),
+	((flags & NSS_INIT_FORCEOPEN) == NSS_INIT_FORCEOPEN), PR_TRUE,
+	((flags & NSS_INIT_OPTIMIZESPACE) == NSS_INIT_OPTIMIZESPACE),
+        ((flags & NSS_INIT_PK11THREADSAFE) == NSS_INIT_PK11THREADSAFE),
+        ((flags & NSS_INIT_PK11RELOAD) == NSS_INIT_PK11RELOAD),
+        ((flags & NSS_INIT_NOPK11FINALIZE) == NSS_INIT_NOPK11FINALIZE));
+    return (rv == SECSuccess) ? context : NULL;
+}
+
 SECStatus
 NSS_InitWithMerge(const char *configdir, const char *certPrefix, 
 	const char *keyPrefix, const char *secmodName, 
@@ -676,7 +818,8 @@ NSS_InitWithMerge(const char *configdir, const char *certPrefix,
 	const char *updateName, PRUint32 flags)
 {
     return nss_Init(configdir, certPrefix, keyPrefix, secmodName,
-	updateDir, updCertPrefix, updKeyPrefix, updateID, updateName,
+	updateDir, updCertPrefix, updKeyPrefix, updateID, updateName, 
+	NULL, NULL,
 	((flags & NSS_INIT_READONLY) == NSS_INIT_READONLY),
 	((flags & NSS_INIT_NOCERTDB) == NSS_INIT_NOCERTDB),
 	((flags & NSS_INIT_NOMODDB) == NSS_INIT_NOMODDB),
@@ -694,7 +837,7 @@ NSS_InitWithMerge(const char *configdir, const char *certPrefix,
 SECStatus
 NSS_NoDB_Init(const char * configdir)
 {
-      return nss_Init("","","","", "", "", "", "", "",
+      return nss_Init("","","","", "", "", "", "", "", NULL, NULL,
 			PR_TRUE,PR_TRUE,PR_TRUE,PR_TRUE,PR_TRUE,PR_TRUE,
 			PR_FALSE,PR_FALSE,PR_FALSE);
 }
@@ -722,7 +865,7 @@ nss_GetShutdownEntry(NSS_ShutdownFunc sFunc, void *appData)
 {
     int count, i;
     count = nssShutdownList.peakFuncs;
-    /* expect the list to be short, just do a linear search */
+
     for (i=0; i < count; i++) {
 	if ((nssShutdownList.funcs[i].func == sFunc) &&
 	    (nssShutdownList.funcs[i].appData == appData)){
@@ -740,8 +883,8 @@ NSS_RegisterShutdown(NSS_ShutdownFunc sFunc, void *appData)
 {
     int i;
 
-    if (!nss_IsInitted) {
-	PORT_SetError(SEC_ERROR_LIBRARY_FAILURE);
+    if (!NSS_IsInitialized()) {
+	PORT_SetError(SEC_ERROR_NOT_INITIALIZED);
 	return SECFailure;
     }
     if (sFunc == NULL) {
@@ -794,8 +937,8 @@ SECStatus
 NSS_UnregisterShutdown(NSS_ShutdownFunc sFunc, void *appData)
 {
     int i;
-    if (!nss_IsInitted) {
-	PORT_SetError(SEC_ERROR_LIBRARY_FAILURE);
+    if (!NSS_IsInitialized()) {
+	PORT_SetError(SEC_ERROR_NOT_INITIALIZED);
 	return SECFailure;
     }
 
@@ -821,6 +964,9 @@ NSS_UnregisterShutdown(NSS_ShutdownFunc sFunc, void *appData)
 static SECStatus
 nss_InitShutdownList(void)
 {
+    if (nssShutdownList.lock != NULL) {
+	return SECSuccess;
+    }
     nssShutdownList.lock = PZ_NewLock(nssILockOther);
     if (nssShutdownList.lock == NULL) {
 	return SECFailure;
@@ -869,16 +1015,12 @@ nss_ShutdownShutdownList(void)
 extern const NSSError NSS_ERROR_BUSY;
 
 SECStatus
-NSS_Shutdown(void)
+nss_Shutdown(void)
 {
     SECStatus shutdownRV = SECSuccess;
     SECStatus rv;
     PRStatus status;
-
-    if (!nss_IsInitted) {
-	PORT_SetError(SEC_ERROR_NOT_INITIALIZED);
-	return SECFailure;
-    }
+    NSSInitContext *temp;
 
     rv = nss_ShutdownShutdownList();
     if (rv != SECSuccess) {
@@ -891,6 +1033,7 @@ NSS_Shutdown(void)
     SECOID_Shutdown();
     status = STAN_Shutdown();
     cert_DestroySubjectKeyIDHashTable();
+    pk11_SetInternalKeySlot(NULL);
     rv = SECMOD_Shutdown();
     if (rv != SECSuccess) {
 	shutdownRV = SECFailure;
@@ -911,14 +1054,87 @@ NSS_Shutdown(void)
 	}
 	shutdownRV = SECFailure;
     }
-    nss_IsInitted = PR_FALSE;
+    nssIsInitted = PR_FALSE;
+    temp = nssInitContextList;
+    nssInitContextList = NULL;
+    /* free the old list. This is necessary when we are called from
+     * NSS_Shutdown(). */
+    while (temp) {
+	NSSInitContext *next = temp->next;
+	temp->magic = 0;
+	PORT_Free(temp);
+	temp = next;
+    }
     return shutdownRV;
 }
 
+SECStatus
+NSS_Shutdown(void)
+{
+    if (!nssIsInitted) {
+	PORT_SetError(SEC_ERROR_NOT_INITIALIZED);
+	return SECFailure;
+    }
+
+    return nss_Shutdown();
+}
+
+/*
+ * remove the context from a list. return true if found, false if not
+ */
+PRBool
+nss_RemoveList(NSSInitContext *context) {
+    NSSInitContext *this = nssInitContextList;
+    NSSInitContext **last = &nssInitContextList;
+
+    while (this) {
+	if (this == context) {
+	    *last = this->next;
+	    this->magic = 0;
+	    PORT_Free(this);
+	    return PR_TRUE;
+	}
+	last = &this->next;
+	this=this->next;
+    }
+    return PR_FALSE;
+}
+
+/*
+ * This form of shutdown is safe in the case where we may have multiple 
+ * entities using NSS in a single process. Each entity calls shutdown with
+ * it's own context. The application (which doesn't get a context), calls
+ * shutdown with NULL. Once all users have 'checked in' NSS will shutdown.
+ * This is different than NSS_Shutdown, where calling it will shutdown NSS
+ * irreguardless of who else may have NSS open.
+ */
+SECStatus
+NSS_ShutdownContext(NSSInitContext *context)
+{
+   if (!context) {
+	if (!nssIsInitted) {
+	    PORT_SetError(SEC_ERROR_NOT_INITIALIZED);
+	    return SECFailure;
+	}
+	nssIsInitted = 0;
+    } else if (! nss_RemoveList(context)) {
+	/* context was already freed or wasn't valid */
+	PORT_SetError(SEC_ERROR_NOT_INITIALIZED);
+	return SECFailure;
+    }
+    if ((nssIsInitted == 0) && (nssInitContextList == NULL)) {
+	return nss_Shutdown();
+    }
+    return SECSuccess;
+}
+	
+	
+
+
 PRBool
 NSS_IsInitialized(void)
 {
-    return nss_IsInitted;
+    return (nssIsInitted) || (nssInitContextList != NULL);
 }
 
 
@@ -935,9 +1151,9 @@ NSS_VersionCheck(const char *importedVersion)
      * check algorithm.  This release is not backward
      * compatible with previous major releases.  It is
      * not compatible with future major, minor, or
-     * patch releases.
+     * patch releases or builds.
      */
-    int vmajor = 0, vminor = 0, vpatch = 0;
+    int vmajor = 0, vminor = 0, vpatch = 0, vbuild = 0;
     const char *ptr = importedVersion;
     volatile char c; /* force a reference that won't get optimized away */
 
@@ -959,6 +1175,13 @@ NSS_VersionCheck(const char *importedVersion)
                 vpatch = 10 * vpatch + *ptr - '0';
                 ptr++;
             }
+            if (*ptr == '.') {
+                ptr++;
+                while (isdigit(*ptr)) {
+                    vbuild = 10 * vbuild + *ptr - '0';
+                    ptr++;
+                }
+            }
         }
     }
 
@@ -971,6 +1194,10 @@ NSS_VersionCheck(const char *importedVersion)
     if (vmajor == NSS_VMAJOR && vminor == NSS_VMINOR && vpatch > NSS_VPATCH) {
         return PR_FALSE;
     }
+    if (vmajor == NSS_VMAJOR && vminor == NSS_VMINOR &&
+        vpatch == NSS_VPATCH && vbuild > NSS_VBUILD) {
+        return PR_FALSE;
+    }
     /* Check dependent libraries */
     if (PR_VersionCheck(PR_VERSION) == PR_FALSE) {
         return PR_FALSE;
