@@ -45,10 +45,8 @@
 
 extern "C" {
 #include <libgnomevfs/gnome-vfs.h>
-#include <libgnomevfs/gnome-vfs-application-registry.h>
 #include <libgnomevfs/gnome-vfs-mime.h>
 #include <libgnomevfs/gnome-vfs-mime-handlers.h>
-#include <libgnomevfs/gnome-vfs-mime-info.h>
 }
 
 class nsGnomeVFSMimeApp : public nsIGnomeVFSMimeApp
@@ -226,24 +224,6 @@ nsGnomeVFSService::GetAppForMimeType(const nsACString &aMimeType,
   return NS_OK;
 }
 
-NS_IMETHODIMP
-nsGnomeVFSService::SetAppForMimeType(const nsACString &aMimeType,
-                                     const nsACString &aId)
-{
-  gnome_vfs_mime_set_default_application(PromiseFlatCString(aMimeType).get(),
-                                         PromiseFlatCString(aId).get());
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsGnomeVFSService::SetIconForMimeType(const nsACString &aMimeType,
-                                      const nsACString &aIconName)
-{
-  gnome_vfs_mime_set_icon(PromiseFlatCString(aMimeType).get(),
-                          PromiseFlatCString(aIconName).get());
-  return NS_OK;
-}
-
 NS_IMETHODIMP
 nsGnomeVFSService::GetDescriptionForMimeType(const nsACString &aMimeType,
                                              nsACString& aDescription)
@@ -282,70 +262,3 @@ nsGnomeVFSService::ShowURIForInput(const nsACString &aUri)
     g_free(spec);
   return rv;
 }
-
-NS_IMETHODIMP
-nsGnomeVFSService::SetAppStringKey(const nsACString &aID,
-                                   PRInt32           aKey,
-                                   const nsACString &aValue)
-{
-  const char *key;
-
-  if (aKey == APP_KEY_COMMAND)
-    key = GNOME_VFS_APPLICATION_REGISTRY_COMMAND;
-  else if (aKey == APP_KEY_NAME)
-    key = GNOME_VFS_APPLICATION_REGISTRY_NAME;
-  else if (aKey == APP_KEY_SUPPORTED_URI_SCHEMES)
-    key = "supported_uri_schemes";
-  else if (aKey == APP_KEY_EXPECTS_URIS)
-    key = "expects_uris";
-  else
-    return NS_ERROR_NOT_AVAILABLE;
-
-  gnome_vfs_application_registry_set_value(PromiseFlatCString(aID).get(), key,
-                                           PromiseFlatCString(aValue).get());
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsGnomeVFSService::SetAppBoolKey(const nsACString &aID,
-                                 PRInt32          aKey,
-                                 PRBool           aValue)
-{
-  const char *key;
-
-  if (aKey == APP_KEY_CAN_OPEN_MULTIPLE)
-    key = GNOME_VFS_APPLICATION_REGISTRY_CAN_OPEN_MULTIPLE_FILES;
-  else if (aKey == APP_KEY_REQUIRES_TERMINAL)
-    key = GNOME_VFS_APPLICATION_REGISTRY_REQUIRES_TERMINAL;
-  else
-    return NS_ERROR_NOT_AVAILABLE;
-
-  gnome_vfs_application_registry_set_bool_value(PromiseFlatCString(aID).get(),
-                                                key, aValue);
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsGnomeVFSService::AddMimeType(const nsACString &aID, const nsACString &aType)
-{
-  gnome_vfs_application_registry_add_mime_type(PromiseFlatCString(aID).get(),
-                                              PromiseFlatCString(aType).get());
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsGnomeVFSService::SyncAppRegistry()
-{
-  gnome_vfs_application_registry_sync();
-  return NS_OK;
-}
-
-NS_IMETHODIMP
-nsGnomeVFSService::SetMimeExtensions(const nsACString &aMimeType,
-                                     const nsACString &aExtensionsList)
-{
-  GnomeVFSResult res =
-    gnome_vfs_mime_set_extensions_list(PromiseFlatCString(aMimeType).get(),
-                                    PromiseFlatCString(aExtensionsList).get());
-  return (res == GNOME_VFS_OK) ? NS_OK : NS_ERROR_FAILURE;
-}
