@@ -36,10 +36,11 @@
  * ***** END LICENSE BLOCK ***** */
 
 #include <stdio.h>
+#include <stdlib.h>
 #include "nsString.h"
 #include "nsStringBuffer.h"
 #include "nsReadableUtils.h"
-#include "nsCRT.h"
+#include "nsCRTGlue.h"
 
 namespace TestStrings {
 
@@ -552,17 +553,18 @@ PRBool test_concat_2()
     return PR_FALSE;
   }
 
-#if 0
 PRBool test_concat_3()
   {
-    nsCString a("a"), b("b");
+    nsCString result;
+    nsCString ab("ab"), c("c");
 
-    // THIS DOES NOT COMPILE
-    const nsACString& r = a + b;
+    result = ab + result + c;
+    if (strcmp(result.get(), "abc") == 0)
+      return PR_TRUE;
 
-    return PR_TRUE;
+    printf("[result=%s]\n", result.get());
+    return PR_FALSE;
   }
-#endif
 
 PRBool test_xpidl_string()
   {
@@ -586,7 +588,7 @@ PRBool test_xpidl_string()
       return PR_FALSE;
 
     const char text[] = "hello world";
-    *getter_Copies(a) = nsCRT::strdup(text);
+    *getter_Copies(a) = NS_strdup(text);
     if (strcmp(a, text) != 0)
       return PR_FALSE;
 
@@ -659,6 +661,59 @@ PRBool test_substring()
     return PR_TRUE;
   }
 
+#define test_append(str, int, suffix) \
+  str.Truncate(); \
+  str.AppendInt(suffix = int ## suffix); \
+  if (!str.EqualsLiteral(#int)) { \
+    fputs("Error appending " #int "\n", stderr); \
+    return PR_FALSE; \
+  }
+
+#define test_appends(int, suffix) \
+  test_append(str, int, suffix) \
+  test_append(cstr, int, suffix)
+
+#define test_appendbase(str, prefix, int, suffix, base) \
+  str.Truncate(); \
+  str.AppendInt(suffix = prefix ## int ## suffix, base); \
+  if (!str.EqualsLiteral(#int)) { \
+    fputs("Error appending " #prefix #int "\n", stderr); \
+    return PR_FALSE; \
+  }
+
+#define test_appendbases(prefix, int, suffix, base) \
+  test_appendbase(str, prefix, int, suffix, base) \
+  test_appendbase(cstr, prefix, int, suffix, base)
+
+PRBool test_appendint()
+  {
+    nsString str;
+    nsCString cstr;
+    PRInt32 L;
+    PRUint32 UL;
+    PRInt64 LL;
+    PRUint64 ULL;
+    test_appends(2147483647, L)
+    test_appends(-2147483648, L)
+    test_appends(4294967295, UL)
+    test_appends(9223372036854775807, LL)
+    test_appends(-9223372036854775808, LL)
+    test_appends(18446744073709551615, ULL)
+    test_appendbases(0, 17777777777, L, 8)
+    test_appendbases(0, 20000000000, L, 8)
+    test_appendbases(0, 37777777777, UL, 8)
+    test_appendbases(0, 777777777777777777777, LL, 8)
+    test_appendbases(0, 1000000000000000000000, LL, 8)
+    test_appendbases(0, 1777777777777777777777, ULL, 8)
+    test_appendbases(0x, 7fffffff, L, 16)
+    test_appendbases(0x, 80000000, L, 16)
+    test_appendbases(0x, ffffffff, UL, 16)
+    test_appendbases(0x, 7fffffffffffffff, LL, 16)
+    test_appendbases(0x, 8000000000000000, LL, 16)
+    test_appendbases(0x, ffffffffffffffff, ULL, 16)
+    return PR_TRUE;
+  }
+
 PRBool test_appendint64()
   {
     nsCString str;
@@ -895,6 +950,21 @@ PRBool test_voided_autostr()
     return PR_TRUE;
   }
 
+PRBool test_voided_assignment()
+  {
+    nsCString a, b;
+    b.SetIsVoid(PR_TRUE);
+    a = b;
+    return a.IsVoid() && a.get() == b.get();
+  }
+
+PRBool test_empty_assignment()
+  {
+    nsCString a, b;
+    a = b;
+    return a.get() == b.get();
+  }
+
 struct ToIntegerTest
 {
   const char *str;
@@ -912,11 +982,15 @@ static const ToIntegerTest kToIntegerTests[] = {
 
 PRBool test_string_tointeger()
 {
-  PRInt32 rv;
+  PRInt32 i;
+  nsresult rv;
   for (const ToIntegerTest* t = kToIntegerTests; t->str; ++t) {
     PRInt32 result = nsCAutoString(t->str).ToInteger(&rv, t->radix);
     if (rv != t->rv || result != t->result)
       return PR_FALSE;
+    result = nsCAutoString(t->str).ToInteger(&i, t->radix);
+    if ((nsresult)i != t->rv || result != t->result)
+      return PR_FALSE;
   }
   return PR_TRUE;
 }
@@ -965,6 +1039,173 @@ static PRBool test_parse_string()
          test_parse_string_helper1("  foo", ' ', "foo");
 }
 
+static PRBool test_strip_chars_helper(const PRUnichar* str, const PRUnichar* strip, const nsAString& result, PRUint32 offset=0)
+{
+  nsAutoString tmp(str);
+  nsAString& data = tmp;
+  data.StripChars(strip, offset);
+  return data.Equals(result);
+}
+
+static PRBool test_strip_chars()
+{
+  return test_strip_chars_helper(NS_LITERAL_STRING("foo \r \nbar").get(),
+                                 NS_LITERAL_STRING(" \n\r").get(),
+                                 NS_LITERAL_STRING("foobar")) &&
+         test_strip_chars_helper(NS_LITERAL_STRING("\r\nfoo\r\n").get(),
+                                 NS_LITERAL_STRING(" \n\r").get(),
+                                 NS_LITERAL_STRING("foo")) &&
+         test_strip_chars_helper(NS_LITERAL_STRING("foo").get(),
+                                 NS_LITERAL_STRING(" \n\r").get(),
+                                 NS_LITERAL_STRING("foo")) &&
+         test_strip_chars_helper(NS_LITERAL_STRING("foo").get(),
+                                 NS_LITERAL_STRING("fo").get(),
+                                 NS_LITERAL_STRING("")) &&
+         test_strip_chars_helper(NS_LITERAL_STRING("foo").get(),
+                                 NS_LITERAL_STRING("foo").get(),
+                                 NS_LITERAL_STRING("")) &&
+         test_strip_chars_helper(NS_LITERAL_STRING(" foo").get(),
+                                 NS_LITERAL_STRING(" ").get(),
+                                 NS_LITERAL_STRING(" foo"), 1);
+}
+
+static PRBool test_huge_capacity()
+{
+  nsString a, b, c, d, e, f, g, h, i, j, k, l, m, n;
+  nsCString n1;
+  PRBool fail = PR_FALSE;
+#undef ok
+#define ok(x) { fail |= !(x); }
+
+  ok(a.SetCapacity(1));
+  ok(!a.SetCapacity(nsString::size_type(-1)/2));
+  ok(a.SetCapacity(0));  // free the allocated memory
+
+  ok(b.SetCapacity(1));
+  ok(!b.SetCapacity(nsString::size_type(-1)/2 - 1));
+  ok(b.SetCapacity(0));
+
+  ok(c.SetCapacity(1));
+  ok(!c.SetCapacity(nsString::size_type(-1)/2));
+  ok(c.SetCapacity(0));
+
+  ok(!d.SetCapacity(nsString::size_type(-1)/2 - 1));
+  ok(!d.SetCapacity(nsString::size_type(-1)/2));
+  ok(d.SetCapacity(0));
+
+  ok(!e.SetCapacity(nsString::size_type(-1)/4));
+  ok(!e.SetCapacity(nsString::size_type(-1)/4 + 1));
+  ok(e.SetCapacity(0));
+
+  ok(!f.SetCapacity(nsString::size_type(-1)/2));
+  ok(f.SetCapacity(0));
+
+  ok(!g.SetCapacity(nsString::size_type(-1)/4 + 1000));
+  ok(!g.SetCapacity(nsString::size_type(-1)/4 + 1001));
+  ok(g.SetCapacity(0));
+
+  ok(!h.SetCapacity(nsString::size_type(-1)/4+1));
+  ok(!h.SetCapacity(nsString::size_type(-1)/2));
+  ok(h.SetCapacity(0));
+
+  ok(i.SetCapacity(1));
+  ok(i.SetCapacity(nsString::size_type(-1)/4 - 1000));
+  ok(!i.SetCapacity(nsString::size_type(-1)/4 + 1));
+  ok(i.SetCapacity(0));
+
+  ok(j.SetCapacity(nsString::size_type(-1)/4 - 1000));
+  ok(!j.SetCapacity(nsString::size_type(-1)/4 + 1));
+  ok(j.SetCapacity(0));
+
+  ok(k.SetCapacity(nsString::size_type(-1)/8 - 1000));
+  ok(k.SetCapacity(nsString::size_type(-1)/4 - 1001));
+  ok(k.SetCapacity(nsString::size_type(-1)/4 - 998));
+  ok(!k.SetCapacity(nsString::size_type(-1)/4 + 1));
+  ok(k.SetCapacity(0));
+
+  ok(l.SetCapacity(nsString::size_type(-1)/8));
+  ok(l.SetCapacity(nsString::size_type(-1)/8 + 1));
+  ok(l.SetCapacity(nsString::size_type(-1)/8 + 2));
+  ok(l.SetCapacity(0));
+
+  ok(m.SetCapacity(nsString::size_type(-1)/8 + 1000));
+  ok(m.SetCapacity(nsString::size_type(-1)/8 + 1001));
+  ok(m.SetCapacity(0));
+
+  ok(n.SetCapacity(nsString::size_type(-1)/8+1));
+  ok(!n.SetCapacity(nsString::size_type(-1)/4));
+  ok(n.SetCapacity(0));
+
+  ok(n.SetCapacity(0));
+  ok(n.SetCapacity((nsString::size_type(-1)/2 - sizeof(nsStringBuffer)) / 2 - 2));
+  ok(n.SetCapacity(0));
+  ok(!n.SetCapacity((nsString::size_type(-1)/2 - sizeof(nsStringBuffer)) / 2 - 1));
+  ok(n.SetCapacity(0));
+  ok(n1.SetCapacity(0));
+  ok(n1.SetCapacity((nsCString::size_type(-1)/2 - sizeof(nsStringBuffer)) / 1 - 2));
+  ok(n1.SetCapacity(0));
+  ok(!n1.SetCapacity((nsCString::size_type(-1)/2 - sizeof(nsStringBuffer)) / 1 - 1));
+  ok(n1.SetCapacity(0));
+
+  // Ignore the result if the address space is less than 64-bit because
+  // some of the allocations above will exhaust the address space.  
+  if (sizeof(void*) >= 8) {
+    return !fail;
+  }
+  return PR_TRUE;
+}
+
+static PRBool test_tofloat_helper(const nsString& aStr, float aExpected, PRBool aSuccess)
+{
+  PRInt32 result;
+  return aStr.ToFloat(&result) == aExpected &&
+         aSuccess ? result == NS_OK : result != NS_OK;
+}
+
+static PRBool test_tofloat()
+{
+  return \
+    test_tofloat_helper(NS_LITERAL_STRING("42"), 42.f, PR_TRUE) &&
+    test_tofloat_helper(NS_LITERAL_STRING("42.0"), 42.f, PR_TRUE) &&
+    test_tofloat_helper(NS_LITERAL_STRING("-42"), -42.f, PR_TRUE) &&
+    test_tofloat_helper(NS_LITERAL_STRING("+42"), 42, PR_TRUE) &&
+    test_tofloat_helper(NS_LITERAL_STRING("13.37"), 13.37f, PR_TRUE) &&
+    test_tofloat_helper(NS_LITERAL_STRING("1.23456789"), 1.23456789f, PR_TRUE) &&
+    test_tofloat_helper(NS_LITERAL_STRING("1.98765432123456"), 1.98765432123456f, PR_TRUE) &&
+    test_tofloat_helper(NS_LITERAL_STRING("0"), 0.f, PR_TRUE) &&
+    test_tofloat_helper(NS_LITERAL_STRING("1.e5"), 100000, PR_TRUE) &&
+    test_tofloat_helper(NS_LITERAL_STRING(""), 0.f, PR_FALSE) &&
+    test_tofloat_helper(NS_LITERAL_STRING("42foo"), 42.f, PR_FALSE) &&
+    test_tofloat_helper(NS_LITERAL_STRING("foo"), 0.f, PR_FALSE) &&
+    PR_TRUE;
+}
+
+static PRBool test_todouble_helper(const nsString& aStr, double aExpected, PRBool aSuccess)
+{
+  PRInt32 result;
+  return aStr.ToDouble(&result) == aExpected &&
+         aSuccess ? result == NS_OK : result != NS_OK;
+}
+
+static PRBool test_todouble()
+{
+  return \
+    test_todouble_helper(NS_LITERAL_STRING("42"), 42, PR_TRUE) &&
+    test_todouble_helper(NS_LITERAL_STRING("42.0"), 42, PR_TRUE) &&
+    test_todouble_helper(NS_LITERAL_STRING("-42"), -42, PR_TRUE) &&
+    test_todouble_helper(NS_LITERAL_STRING("+42"), 42, PR_TRUE) &&
+    test_todouble_helper(NS_LITERAL_STRING("13.37"), 13.37, PR_TRUE) &&
+    test_todouble_helper(NS_LITERAL_STRING("1.23456789"), 1.23456789, PR_TRUE) &&
+    test_todouble_helper(NS_LITERAL_STRING("1.98765432123456"), 1.98765432123456, PR_TRUE) &&
+    test_todouble_helper(NS_LITERAL_STRING("123456789.98765432123456"), 123456789.98765432123456, PR_TRUE) &&
+    test_todouble_helper(NS_LITERAL_STRING("0"), 0, PR_TRUE) &&
+    test_todouble_helper(NS_LITERAL_STRING("1.e5"), 100000, PR_TRUE) &&
+    test_todouble_helper(NS_LITERAL_STRING(""), 0, PR_FALSE) &&
+    test_todouble_helper(NS_LITERAL_STRING("42foo"), 42, PR_FALSE) &&
+    test_todouble_helper(NS_LITERAL_STRING("foo"), 0, PR_FALSE) &&
+    PR_TRUE;
+}
+
 //----
 
 typedef PRBool (*TestFunc)();
@@ -997,10 +1238,12 @@ tests[] =
     { "test_fixed_string", test_fixed_string },
     { "test_concat", test_concat },
     { "test_concat_2", test_concat_2 },
+    { "test_concat_3", test_concat_3 },
     { "test_xpidl_string", test_xpidl_string },
     { "test_empty_assign", test_empty_assign },
     { "test_set_length", test_set_length },
     { "test_substring", test_substring },
+    { "test_appendint", test_appendint },
     { "test_appendint64", test_appendint64 },
     { "test_appendfloat", test_appendfloat },
     { "test_findcharinset", test_findcharinset },
@@ -1008,8 +1251,14 @@ tests[] =
     { "test_stringbuffer", test_stringbuffer },
     { "test_voided", test_voided },
     { "test_voided_autostr", test_voided_autostr },
+    { "test_voided_assignment", test_voided_assignment },
+    { "test_empty_assignment", test_empty_assignment },
     { "test_string_tointeger", test_string_tointeger },
     { "test_parse_string", test_parse_string },
+    { "test_strip_chars", test_strip_chars },
+    { "test_huge_capacity", test_huge_capacity },
+    { "test_tofloat", test_tofloat },
+    { "test_todouble", test_todouble },
     { nsnull, nsnull }
   };
 
@@ -1023,6 +1272,8 @@ int main(int argc, char **argv)
     if (argc > 1)
       count = atoi(argv[1]);
 
+    NS_LogInit();
+
     while (count--)
       {
         for (const Test* t = tests; t->name != nsnull; ++t)
